@@ -1,31 +1,28 @@
 import copy
 import os
-import traceback
 
 from PyQt5.QtCore import QDir, Qt, pyqtSlot, QTimer
 from PyQt5.QtGui import QIcon, QCloseEvent, QKeySequence
-from PyQt5.QtWidgets import QMainWindow, QUndoGroup, QActionGroup, QHeaderView, QAction, QFileDialog, \
-    QMessageBox, QApplication
+from PyQt5.QtWidgets import QMainWindow, QUndoGroup, QActionGroup, QHeaderView, QAction, QMessageBox, QApplication, qApp
 
-from urh import constants, version
-from urh.controller.dialogs.CSVImportDialog import CSVImportDialog
+from urh import settings, version
 from urh.controller.CompareFrameController import CompareFrameController
-from urh.controller.dialogs.DecoderDialog import DecoderDialog
 from urh.controller.GeneratorTabController import GeneratorTabController
+from urh.controller.SignalTabController import SignalTabController
+from urh.controller.SimulatorTabController import SimulatorTabController
+from urh.controller.dialogs.CSVImportDialog import CSVImportDialog
+from urh.controller.dialogs.DecoderDialog import DecoderDialog
 from urh.controller.dialogs.OptionsDialog import OptionsDialog
 from urh.controller.dialogs.ProjectDialog import ProjectDialog
 from urh.controller.dialogs.ProtocolSniffDialog import ProtocolSniffDialog
 from urh.controller.dialogs.ReceiveDialog import ReceiveDialog
-from urh.controller.widgets.SignalFrame import SignalFrame
-from urh.controller.SignalTabController import SignalTabController
-from urh.controller.SimulatorTabController import SimulatorTabController
 from urh.controller.dialogs.SpectrumDialogController import SpectrumDialogController
+from urh.controller.widgets.SignalFrame import SignalFrame
 from urh.models.FileFilterProxyModel import FileFilterProxyModel
 from urh.models.FileIconProvider import FileIconProvider
 from urh.models.FileSystemModel import FileSystemModel
 from urh.models.ParticipantLegendListModel import ParticipantLegendListModel
 from urh.plugins.PluginManager import PluginManager
-from urh.signalprocessing.Message import Message
 from urh.signalprocessing.ProtocolAnalyzer import ProtocolAnalyzer
 from urh.signalprocessing.Signal import Signal
 from urh.ui.ui_main import Ui_MainWindow
@@ -82,6 +79,8 @@ class MainController(QMainWindow):
         self.cancel_action.setIcon(QIcon.fromTheme("dialog-cancel"))
         self.addAction(self.cancel_action)
 
+        self.ui.actionAuto_detect_new_signals.setChecked(settings.read("auto_detect_new_signals", True, bool))
+
         self.participant_legend_model = ParticipantLegendListModel(self.project_manager.participants)
         self.ui.listViewParticipants.setModel(self.participant_legend_model)
 
@@ -102,7 +101,7 @@ class MainController(QMainWindow):
 
         self.recentFileActionList = []
         self.create_connects()
-        self.init_recent_file_action_list(constants.SETTINGS.value("recentFiles", []))
+        self.init_recent_file_action_list(settings.read("recentFiles", [], list))
 
         self.filemodel = FileSystemModel(self)
         path = QDir.homePath()
@@ -141,7 +140,7 @@ class MainController(QMainWindow):
         self.ui.splitter.setSizes([0, 1])
         self.refresh_main_menu()
 
-        self.apply_default_view(constants.SETTINGS.value('default_view', type=int))
+        self.apply_default_view(settings.read('default_view', type=int))
         self.project_save_timer.start(ProjectManager.AUTOSAVE_INTERVAL_MINUTES * 60 * 1000)
 
         self.ui.actionProject_settings.setVisible(False)
@@ -149,7 +148,7 @@ class MainController(QMainWindow):
         self.ui.actionClose_project.setVisible(False)
 
     def __set_non_project_warning_visibility(self):
-        show = constants.SETTINGS.value("show_non_project_warning", True, bool) and not self.project_manager.project_loaded
+        show = settings.read("show_non_project_warning", True, bool) and not self.project_manager.project_loaded
         self.ui.labelNonProjectMode.setVisible(show)
 
     def create_connects(self):
@@ -179,6 +178,7 @@ class MainController(QMainWindow):
         self.ui.actionSniff_protocol.triggered.connect(self.show_proto_sniff_dialog)
         self.ui.actionAbout_Qt.triggered.connect(QApplication.instance().aboutQt)
         self.ui.actionSamples_from_csv.triggered.connect(self.on_import_samples_from_csv_action_triggered)
+        self.ui.actionAuto_detect_new_signals.triggered.connect(self.on_auto_detect_new_signals_action_triggered)
 
         self.ui.btnFileTreeGoUp.clicked.connect(self.on_btn_file_tree_go_up_clicked)
         self.ui.fileTree.directory_open_wanted.connect(self.project_manager.set_project_folder)
@@ -204,6 +204,7 @@ class MainController(QMainWindow):
             self.on_show_field_types_config_action_triggered)
 
         self.compare_frame_controller.load_protocol_clicked.connect(self.on_compare_frame_controller_load_protocol_clicked)
+        self.compare_frame_controller.ui.listViewParticipants.doubleClicked.connect(self.on_project_settings_action_triggered)
 
         self.ui.lnEdtTreeFilter.textChanged.connect(self.on_file_tree_filter_text_changed)
 
@@ -225,7 +226,7 @@ class MainController(QMainWindow):
         self.ui.labelNonProjectMode.linkActivated.connect(self.on_label_non_project_mode_link_activated)
 
         self.ui.menuFile.addSeparator()
-        for i in range(constants.MAX_RECENT_FILE_NR):
+        for i in range(settings.MAX_RECENT_FILE_NR):
             recent_file_action = QAction(self)
             recent_file_action.setVisible(False)
             recent_file_action.triggered.connect(self.on_open_recent_action_triggered)
@@ -253,6 +254,7 @@ class MainController(QMainWindow):
         proto = self.compare_frame_controller.add_protocol_from_file(filename)
         if proto:
             self.__add_empty_frame_for_filename(proto, filename)
+        self.ui.tabWidget.setCurrentWidget(self.ui.tab_protocol)
 
     def add_fuzz_profile(self, filename):
         self.ui.tabWidget.setCurrentIndex(2)
@@ -272,7 +274,7 @@ class MainController(QMainWindow):
         sig_name = os.path.splitext(os.path.basename(filename))[0]
 
         # Use default sample rate for signal
-        # Sample rate will be overriden in case of a project later
+        # Sample rate will be overridden in case of a project later
         if enforce_sample_rate is not None:
             sample_rate = enforce_sample_rate
         else:
@@ -292,14 +294,22 @@ class MainController(QMainWindow):
         signal.blockSignals(True)
         has_entry = self.project_manager.read_project_file_for_signal(signal)
 
-        if not has_entry and not signal.changed:
-            signal.auto_detect()
+        if self.ui.actionAuto_detect_new_signals.isChecked() and not has_entry and not signal.changed:
+            sig_frame.ui.stackedWidget.setCurrentWidget(sig_frame.ui.pageLoading)
+            qApp.processEvents()
+            signal.auto_detect(detect_modulation=True, detect_noise=False)
+            sig_frame.ui.stackedWidget.setCurrentWidget(sig_frame.ui.pageSignal)
 
         signal.blockSignals(False)
 
         self.signal_protocol_dict[sig_frame] = pa
 
-        sig_frame.refresh(draw_full_signal=True)  # protocol is derived here
+        sig_frame.refresh_signal(draw_full_signal=True)
+        sig_frame.refresh_signal_information(block=True)
+
+        qApp.processEvents()
+        sig_frame.show_protocol(refresh=True)
+
         if self.project_manager.read_participants_for_signal(signal, pa.messages):
             sig_frame.ui.gvSignal.redraw_view()
 
@@ -343,7 +353,7 @@ class MainController(QMainWindow):
             self.set_frame_numbers()
             self.refresh_main_menu()
         except Exception as e:
-            Errors.generic_error(self.tr("Failed to close"), str(e), traceback.format_exc())
+            Errors.exception(e)
             self.unsetCursor()
 
     def add_files(self, filepaths, group_id=0, enforce_sample_rate=None):
@@ -389,7 +399,7 @@ class MainController(QMainWindow):
             elif filename.endswith(".csv"):
                 self.__import_csv(filename, group_id)
                 continue
-            elif os.path.basename(filename) == constants.PROJECT_FILE:
+            elif os.path.basename(filename) == settings.PROJECT_FILE:
                 self.project_manager.set_project_folder(os.path.split(filename)[0])
             else:
                 self.add_signalfile(filename, group_id, enforce_sample_rate=enforce_sample_rate)
@@ -499,16 +509,15 @@ class MainController(QMainWindow):
         if file_path in FileOperator.archives.keys():
             file_path = copy.copy(FileOperator.archives[file_path])
 
-        settings = constants.SETTINGS
-        recent_file_paths = settings.value("recentFiles", [])
+        recent_file_paths = settings.read("recentFiles", [], list)
         recent_file_paths = [] if recent_file_paths is None else recent_file_paths  # check None for OSX
         recent_file_paths = [p for p in recent_file_paths if p != file_path and p is not None and os.path.exists(p)]
         recent_file_paths.insert(0, file_path)
-        recent_file_paths = recent_file_paths[:constants.MAX_RECENT_FILE_NR]
+        recent_file_paths = recent_file_paths[:settings.MAX_RECENT_FILE_NR]
 
         self.init_recent_file_action_list(recent_file_paths)
 
-        settings.setValue("recentFiles", recent_file_paths)
+        settings.write("recentFiles", recent_file_paths)
 
     def init_recent_file_action_list(self, recent_file_paths: list):
         for i in range(len(self.recentFileActionList)):
@@ -551,7 +560,7 @@ class MainController(QMainWindow):
                 self.add_files(FileOperator.uncompress_archives([action.data()], QDir.tempPath()))
                 self.unsetCursor()
         except Exception as e:
-            Errors.generic_error(self.tr("Failed to open"), str(e), traceback.format_exc())
+            Errors.exception(e)
             self.unsetCursor()
 
     @pyqtSlot()
@@ -649,6 +658,9 @@ class MainController(QMainWindow):
                 self.generator_tab_controller.ui.tabWidget.widget(i).layout().setContentsMargins(0, 7 + h - th, 0, 0)
             # Modulators may got changed from Simulator Dialog
             self.generator_tab_controller.refresh_modulators()
+            # Signals may got reordered in analysis
+            self.generator_tab_controller.tree_model.update()
+            self.generator_tab_controller.ui.treeProtocols.expandAll()
 
     @pyqtSlot()
     def on_show_record_dialog_action_triggered(self):
@@ -771,7 +783,7 @@ class MainController(QMainWindow):
                     self.add_files(file_names)
                     self.unsetCursor()
             except Exception as e:
-                Errors.generic_error(self.tr("Failed to open"), str(e), traceback.format_exc())
+                Errors.exception(e)
                 self.unsetCursor()
 
     @pyqtSlot()
@@ -863,6 +875,10 @@ class MainController(QMainWindow):
     def on_import_samples_from_csv_action_triggered(self):
         self.__import_csv(file_name="")
 
+    @pyqtSlot(bool)
+    def on_auto_detect_new_signals_action_triggered(self, checked: bool):
+        settings.write("auto_detect_new_signals", bool(checked))
+
     def __import_csv(self, file_name, group_id=0):
         def on_data_imported(complex_file, sample_rate):
             sample_rate = None if sample_rate == 0 else sample_rate
@@ -876,7 +892,7 @@ class MainController(QMainWindow):
     def on_label_non_project_mode_link_activated(self, link: str):
         if link == "dont_show_non_project_again":
             self.ui.labelNonProjectMode.hide()
-            constants.SETTINGS.setValue("show_non_project_warning", False)
+            settings.write("show_non_project_warning", False)
         elif link == "open_new_project_dialog":
             self.on_new_project_action_triggered()
 
